@@ -49,7 +49,7 @@ class Stage(Gtk.Window):
         self.get_style_context().add_class("csstage")
 
         trackers.con_tracker_get().connect(singletons.Backgrounds,
-                                           "changed", 
+                                           "changed",
                                            self.on_bg_changed)
 
         self.destroying = False
@@ -67,6 +67,8 @@ class Stage(Gtk.Window):
         self.audio_panel = None
         self.info_panel = None
 
+        self.stage_refresh_id = 0
+
         self.floaters = []
 
         self.event_handler = EventHandler(manager)
@@ -106,7 +108,7 @@ class Stage(Gtk.Window):
         self.add(self.overlay)
 
         # We hang onto the UPowerClient here so power events can
-        # trigger changes to and from low-power mode (no plugins.)
+        # trigger changes to the info panel.
         self.power_client = singletons.UPowerClient
 
         trackers.con_tracker_get().connect(self.power_client,
@@ -144,32 +146,63 @@ class Stage(Gtk.Window):
             print("Problem updating monitor views views: %s" % str(e))
 
     def on_screen_size_changed(self, screen, data=None):
-        if status.Debug:
-            print("Stage: Received screen changed signal, updating backdrop")
+        """
+        The screen changing size should be acted upon immediately, to ensure coverage.
+        Wallpapers are secondary.
+        """
 
-        Gdk.flush()
+        if status.Debug:
+            print("Stage: Received screen size-changed signal, refreshing stage")
 
         self.update_geometry()
         self.move_onscreen()
-
         self.overlay.queue_resize()
 
+
     def on_monitors_changed(self, screen, data=None):
+        """
+        Updating monitors also will trigger an immediate stage coverage update (same
+        as on_screen_size_changed), and follow up at idle with actual monitor view
+        refreshes (wallpapers.)
+        """
         if status.Debug:
-            print("Stage: Received screen monitors-changed signal, updating monitor views")
+            print("Stage: Received screen monitors-changed signal, refreshing stage")
+
+        self.update_geometry()
+        self.move_onscreen()
+        self.overlay.queue_resize()
 
         Gdk.flush()
 
-        self.update_monitors()
-        self.overlay.queue_resize()
+        self.queue_refresh_stage()
 
     def on_grab_broken_event(self, widget, event, data=None):
         GObject.idle_add(self.manager.grab_stage)
 
         return False
 
-    def refresh(self):
+    def queue_refresh_stage(self):
+        """
+        Queues a complete refresh of the stage, resizing the screen if necessary,
+        reconstructing the individual monitor objects, etc...
+        """
+        if self.stage_refresh_id > 0:
+            GObject.source_remove(self.stage_refresh_id)
+            self.stage_refresh_id = 0
+
+        self.stage_refresh_id = GObject.idle_add(self._update_full_stage_on_idle)
+
+    def _update_full_stage_on_idle(self, data=None):
+        self.stage_refresh_id = 0
+
+        self._refresh()
+
+        return False
+
+    def _refresh(self):
         Gdk.flush()
+        if status.Debug:
+            print("Stage: refresh callback")
 
         self.update_geometry()
         self.move_onscreen()
@@ -231,6 +264,14 @@ class Stage(Gtk.Window):
         self.gdk_filter.start(self)
 
     def move_onscreen(self):
+        w = self.get_window()
+
+        if w:
+            w.move_resize(self.rect.x,
+                          self.rect.y,
+                          self.rect.width,
+                          self.rect.height)
+
         self.move(self.rect.x, self.rect.y)
         self.resize(self.rect.width, self.rect.height)
 
@@ -369,9 +410,12 @@ class Stage(Gtk.Window):
                                               "monitors-changed",
                                               self.on_monitors_changed)
 
-        status.screen = None
+        trackers.con_tracker_get().disconnect(self.overlay,
+                                              "get-child-position",
+                                              self.position_overlay_child)
 
         self.destroy()
+        status.screen = None
 
     def setup_monitors(self):
         """
@@ -415,8 +459,8 @@ class Stage(Gtk.Window):
             image = Gtk.Image()
 
             singletons.Backgrounds.create_and_set_gtk_image (image,
-                                                  monitor.rect.width,
-                                                  monitor.rect.height)
+                                                             monitor.rect.width,
+                                                             monitor.rect.height)
 
             monitor.set_next_wallpaper_image(image)
 
@@ -424,26 +468,11 @@ class Stage(Gtk.Window):
         """
         Callback for UPower changes, this will make our MonitorViews update
         themselves according to user setting and power state.
-
-        This is in two parts - it looks nicer to reveal/hide the info panel
-        only after the MonitorView changes, so we attach to a MonitorView signal
-        temporarily, which tells us then any animation is complete.
         """
-        trackers.con_tracker_get().connect(self.monitors[0],
-                                           "current-view-change-complete",
-                                           self.after_power_state_changed)
+        if status.Debug:
+            print("stage: Power state changed, updating info panel")
 
-        self.update_monitor_views()
-
-    def after_power_state_changed(self, monitor):
-        """
-        Update the visibility of the InfoPanel after updating the MonitorViews
-        """
-        trackers.con_tracker_get().disconnect(monitor,
-                                              "current-view-change-complete",
-                                              self.after_power_state_changed)
-
-        self.info_panel.update_revealed()
+        self.info_panel.update_visibility()
 
     def setup_clock(self):
         """
@@ -454,14 +483,13 @@ class Stage(Gtk.Window):
         Initially invisible, regardless - its visibility is controlled via its
         own positioning timer.
         """
-        self.clock_widget = ClockWidget(self.away_message, status.screen.get_mouse_monitor())
+        self.clock_widget = ClockWidget(self.away_message, status.screen.get_mouse_monitor(), status.screen.get_low_res_mode())
         self.add_child_widget(self.clock_widget)
 
         self.floaters.append(self.clock_widget)
 
-        if (not settings.should_show_plugin()) or (settings.should_show_plugin() and not self.power_client.plugged_in):
-            if settings.get_show_clock():
-                self.clock_widget.start_positioning()
+        if settings.get_show_clock():
+            self.clock_widget.start_positioning()
 
     def setup_albumart(self):
         """
@@ -477,9 +505,8 @@ class Stage(Gtk.Window):
 
         self.floaters.append(self.clock_widget)
 
-        if (not settings.should_show_plugin()) or (settings.should_show_plugin() and not self.power_client.plugged_in):
-            if settings.get_show_albumart():
-                self.albumart_widget.start_positioning()
+        if settings.get_show_albumart():
+            self.albumart_widget.start_positioning()
 
     def setup_unlock(self):
         """
@@ -529,6 +556,8 @@ class Stage(Gtk.Window):
         self.info_panel = InfoPanel()
         self.add_child_widget(self.info_panel)
 
+        self.info_panel.update_visibility()
+
     def queue_dialog_key_event(self, event):
         """
         Sent from our EventHandler via the ScreensaverManager, this catches
@@ -601,10 +630,6 @@ class Stage(Gtk.Window):
     def raise_unlock_widget(self):
         """
         Bring the unlock widget to the front and make sure it's visible.
-
-        This is done in two steps - we don't want to show anything over a plugin
-        (graphic glitches abound) - so we update the MonitorViews first, then do
-        our other reveals after its transition is complete.
         """
         self.reset_timeout()
 
@@ -620,35 +645,20 @@ class Stage(Gtk.Window):
 
         status.Awake = True
 
-        # Connect to one of our monitorViews (we have at least one always), to wait for
-        # its transition to finish before running after_wallpaper_shown_for_unlock()
-
-        if len(self.monitors) > 0:
-            trackers.con_tracker_get().connect(self.monitors[0],
-                                               "current-view-change-complete",
-                                               self.after_wallpaper_shown_for_unlock)
-
-        self.update_monitor_views()
-
-    def after_wallpaper_shown_for_unlock(self, monitor, data=None):
-        """
-        Finish raising the unlock widget - also bring up our status bars if applicable.
-        """
-        trackers.con_tracker_get().disconnect(monitor,
-                                              "current-view-change-complete",
-                                              self.after_wallpaper_shown_for_unlock)
+        if self.info_panel:
+            self.info_panel.refresh_power_state()
 
         if self.clock_widget != None:
-            self.clock_widget.reveal()
+            self.clock_widget.show()
         if self.albumart_widget != None:
-            self.albumart_widget.reveal()
+            self.albumart_widget.show()
 
-        self.unlock_dialog.reveal()
+        self.unlock_dialog.show()
 
         if self.audio_panel != None:
-            self.audio_panel.reveal()
+            self.audio_panel.show_panel()
         if self.info_panel != None:
-            self.info_panel.update_revealed()
+            self.info_panel.update_visibility()
 
     def cancel_unlocking(self):
         if self.unlock_dialog:
@@ -657,11 +667,6 @@ class Stage(Gtk.Window):
     def cancel_unlock_widget(self):
         """
         Hide the unlock widget (and others) if the unlock has been canceled
-
-        This process is in three steps for aesthetic reasons - 
-            a) Unreveal all widgets (begin fading them out)
-            b) Switch over MonitorViews from wallpaper to plug-ins if needed.
-            c) Re-reveal the InfoPanel if applicable
         """
         if not status.Awake:
             return
@@ -669,90 +674,43 @@ class Stage(Gtk.Window):
         self.set_timeout_active(None, False)
         utils.clear_clipboards(self.unlock_dialog)
 
-        trackers.con_tracker_get().connect(self.unlock_dialog,
-                                           "notify::child-revealed",
-                                           self.after_unlock_unrevealed)
-        self.unlock_dialog.unreveal()
-
-        if self.clock_widget != None:
-            self.clock_widget.unreveal()
-        if self.albumart_widget != None:
-            self.albumart_widget.unreveal()
-        if self.audio_panel != None:
-            self.audio_panel.unreveal()
-        if self.info_panel != None:
-            self.info_panel.unreveal()
-
-    def after_unlock_unrevealed(self, obj, pspec):
-        """
-        Called after unlock unreveal is complete.  Tells the MonitorViews
-        to update themselves.
-        """
         self.unlock_dialog.hide()
-        self.unlock_dialog.cancel()
 
-        if self.audio_panel != None:
-            self.audio_panel.hide()
         if self.clock_widget != None:
             self.clock_widget.hide()
         if self.albumart_widget != None:
             self.albumart_widget.hide()
+        if self.audio_panel != None:
+            self.audio_panel.hide()
+        if self.info_panel != None:
+            self.info_panel.hide()
 
-        trackers.con_tracker_get().disconnect(self.unlock_dialog,
-                                              "notify::child-revealed",
-                                              self.after_unlock_unrevealed)
-
+        self.unlock_dialog.cancel()
         status.Awake = False
 
-        trackers.con_tracker_get().connect(self.monitors[0],
-                                           "current-view-change-complete",
-                                           self.after_transitioned_back_to_sleep)
-
         self.update_monitor_views()
-
-    def after_transitioned_back_to_sleep(self, monitor, data=None):
-        """
-        Called after the MonitorViews have updated - re-show the clock (if desired)
-        and the InfoPanel (if required.)
-        """
-        trackers.con_tracker_get().disconnect(monitor,
-                                              "current-view-change-complete",
-                                              self.after_transitioned_back_to_sleep)
-
-        self.info_panel.update_revealed()
+        self.info_panel.update_visibility()
 
     def update_monitor_views(self):
         """
         Updates all of our MonitorViews based on the power
         or Awake states.
         """
-        low_power = not self.power_client.plugged_in
+
         if not status.Awake:
-            if (not settings.should_show_plugin()) or (settings.should_show_plugin() and low_power):
-                if self.clock_widget != None and settings.get_show_clock():
-                    self.clock_widget.start_positioning()
-                if self.albumart_widget != None and settings.get_show_albumart():
-                    self.albumart_widget.start_positioning()
-            else:
-                if self.clock_widget != None:
-                    self.clock_widget.stop_positioning()
-                    self.clock_widget.hide()
-                if self.albumart_widget != None:
-                    self.albumart_widget.stop_positioning()
-                    self.albumart_widget.hide()
+            if self.clock_widget != None and settings.get_show_clock():
+                self.clock_widget.start_positioning()
+            if self.albumart_widget != None and settings.get_show_albumart():
+                self.albumart_widget.start_positioning()
 
         for monitor in self.monitors:
-            monitor.update_view(status.Awake, low_power)
-
-            if not monitor.get_reveal_child():
-                monitor.reveal()
+                monitor.show()
 
     def destroy_monitor_views(self):
         """
         Destroy all MonitorViews
         """
         for monitor in self.monitors:
-            monitor.kill_plugin()
             monitor.destroy()
             del monitor
 
@@ -901,7 +859,7 @@ class Stage(Gtk.Window):
             monitor_rect = status.screen.get_monitor_geometry(current_monitor)
 
             region_w = monitor_rect.width / 3
-            region_h = monitor_rect.height / 3
+            region_h = monitor_rect.height
 
             if status.Awake:
                 """
@@ -917,6 +875,8 @@ class Stage(Gtk.Window):
                 if (unlock_nw > region_w):
                     region_w = (monitor_rect.width - unlock_nw) / 2
 
+                region_h = monitor_rect.height
+
                 if isinstance(child, ClockWidget):
                     child.set_halign(Gtk.Align.START)
                 else:
@@ -934,6 +894,8 @@ class Stage(Gtk.Window):
                         if floater.get_halign() != child.get_halign() and floater.get_valign() != child.get_valign():
                             continue
 
+                        region_h = monitor_rect.height / 3
+
                         fa = floater.get_halign()
                         ca = child.get_halign()
                         while fa == ca:
@@ -997,8 +959,7 @@ class Stage(Gtk.Window):
 
         if isinstance(child, InfoPanel):
             """
-            The InfoPanel can be shown while not Awake, but only if we're not running
-            a screensaver plugin.  In any case, it will only appear if a) We have received
+            The InfoPanel can be shown while not Awake, but will only appear if a) We have received
             notifications while the screensaver is running, or b) we're either on battery
             or plugged in but with a non-full battery.  It attaches itself to the upper-right
             corner of the monitor.
